/*
 * Daneel - Dalvik to Java bytecode compiler
 * Copyright (C) 2011  IcedRobot team
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * This file is subject to the "Classpath" exception:
 *
 * Linking this library statically or dynamically with other modules is
 * making a combined work based on this library.  Thus, the terms and
 * conditions of the GNU General Public License cover the whole
 * combination.
 *
 * As a special exception, the copyright holders of this library give you
 * permission to link this library with independent modules to produce an
 * executable, regardless of the license terms of these independent
 * modules, and to copy and distribute the resulting executable under terms
 * of your choice, provided that you also meet, for each linked independent
 * module, the terms and conditions of the license of that module.  An
 * independent module is a module which is not derived from or based on
 * this library.  If you modify this library, you may extend this exception
 * to your version of the library, but you are not obligated to do so.  If
 * you do not wish to do so, delete this exception statement from your
 * version.
 */

package org.icedrobot.daneel.rewriter;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertSame;

import org.icedrobot.daneel.DexifyingRunner;
import org.junit.Test;
import org.junit.runner.RunWith;

@RunWith(DexifyingRunner.class)
public class FieldStaticTest {

    @Test
    public void testPresence() throws Exception {
        Class<?> cls = DEXCode.class;
        String fmt = "public static %s " + cls.getName() + ".%s";
        assertEquals(7, cls.getFields().length);
        assertEquals(String.format(fmt, "byte", "staticByte"),
                cls.getField("staticByte").toString());
        assertEquals(String.format(fmt, "short", "staticShort"),
                cls.getField("staticShort").toString());
        assertEquals(String.format(fmt, "int", "staticInt"),
                cls.getField("staticInt").toString());
        assertEquals(String.format(fmt, "long", "staticLong"),
                cls.getField("staticLong").toString());
        assertEquals(String.format(fmt, "float", "staticFloat"),
                cls.getField("staticFloat").toString());
        assertEquals(String.format(fmt, "double", "staticDouble"),
                cls.getField("staticDouble").toString());
        assertEquals(String.format(fmt, "java.lang.Object", "staticObject"),
                cls.getField("staticObject").toString());
    }

    @Test
    public void testWriteByte() {
        assertEquals(0, DEXCode.staticByte);
        DEXCode.staticByte = 23;
        assertEquals(23, DEXCode.staticByte);
    }

    @Test
    public void testWriteShort() {
        assertEquals(0, DEXCode.staticShort);
        DEXCode.staticShort = 23;
        assertEquals(23, DEXCode.staticShort);
    }

    @Test
    public void testWriteInt() {
        assertEquals(0, DEXCode.staticInt);
        DEXCode.staticInt = 23;
        assertEquals(23, DEXCode.staticInt);
    }

    @Test
    public void testWriteLong() {
        assertEquals(0, DEXCode.staticLong);
        DEXCode.staticLong = 23;
        assertEquals(23, DEXCode.staticLong);
    }

    @Test
    public void testWriteFloat() {
        assertEquals(0, DEXCode.staticFloat, 0);
        DEXCode.staticFloat = 2.3f;
        assertEquals(2.3f, DEXCode.staticFloat, 0);
    }

    @Test
    public void testWriteDouble() {
        assertEquals(0, DEXCode.staticDouble, 0);
        DEXCode.staticDouble = 2.3;
        assertEquals(2.3, DEXCode.staticDouble, 0);
    }

    @Test
    public void testWriteObject() {
        assertNull(DEXCode.staticObject);
        Object obj = new String("some test object");
        DEXCode.staticObject = obj;
        assertSame(obj, DEXCode.staticObject);
    }

    // Keep this class named "DEXCode" to push it through Daneel.
    private static class DEXCode {
        public static byte staticByte;
        public static short staticShort;
        public static int staticInt;
        public static long staticLong;
        public static float staticFloat;
        public static double staticDouble;
        public static Object staticObject;
    };
}
